/*
  Name : main.cpp
  Copyright : KPAMEGAN, ROUSSET, MARTY & FOK
  Authors : KPAMEGAN Karel, ROUSSET Baptiste, MARTY Sbastien & FOK Pierre
  Description: main file of the automaton
*/

#include "temps.h"
#include <math.h>
#include <fcntl.h>
#include <iomanip>
#include <fstream>
#include "automaton.hpp"


#include "configuration.hpp"
#include "alert.hpp"
#include "shareData.hpp"
#include "routine.hpp"
#include "initTimer.hpp"

//BD Connection
#define HOST "localhost"
#define BASE "automate"
#define LOGIN "root"
#define PASS ""


/****************************************/
// used for getting the k8055 config
config cfg; 


/************SCREEN PARAMETERS*******************/

//extern Alert alert;        //the alerts screen which comes from routine.cpp
//extern Alert logs;         //the logs screen which comes from routine.cpp
pthread_mutex_t mut;     //variable of exclusion, use in routine.cpp and other.cpp
shareData dataVal;       //the structure which contents the aquarium configuration, use in routine.cpp and other.cpp
InitTimes confTimes(4,3,0); //initialize the time between 2 screens, the number of blinking of ever title and the state of the logs

//*********************************************************
int main(int argc, char** argv)
{
  connexion c(HOST,BASE,LOGIN,PASS);  
  short st,sh,day;
 bool connected=true; // used with bd connection
  bool new_m=false;// used to check new hour
  double t2,flow=0.0,ph;
  double temperature=25;
  double tempe=0;
  char temp[100];
  
  double te[20]; 
  int count=0; // used to compute the temperature from 10 values retains the best one !!!
  string s,s2,m;
  char ch=0;
  


  //******************************************************************
      if(argc > 1)
      {	
	if(!strcmp(argv[1],"init-bd") )
	  {
	    c.construct("automate.sql");
	    // database connection
	    c.connect_to_db();
	    insert_error(c,2);
	  }
	if(!strcmp(argv[1],"init-bd2") )
	  {
	    c.construct("automate2.sql");
	    // database connection
	    c.connect_to_db();
	    insert_error(c,2);
	  }

	}
	else 
		{
		connected=c.connect_to_db(); // database connection
		if (connected == false)
			{
			// bd error
			c.construct("automate.sql");
	    		// database connection
	    		connected=c.connect_to_db();
			if (connected== false)
				{
				//2d bd error
				c.construct("automate2.sql");
	    			// database connection
	    			connected=c.connect_to_db();
				if (connected==false) exit(0); //3rd bd error
				else insert_error(c,3);
				}
			else insert_error(c,2);
			}
  		}
  //fcntl(0,F_SETFL,O_NONBLOCK); //deblocking stdin
  sleep(2); // used to wait the end of another automaton process
  
 
  // get the config
  c.get_config(cfg);
  c.init_out();
  
if (cfg.LCD && cfg.LOGLCD) confTimes.set_state_log(true); else confTimes.set_state_log(false); //if lcd present and if log must be written, configure the lcd screen

K8055 k(c); //card obj
Automaton a; // automaton obj
  
  // init start to 0 in database models and indicates the starting of the automaton
  c.set_start();
/********************************************************************/
  if (cfg.LCD) // if the lcd screen isn't present, we don't create this thread
  {
    pthread_t th;
    pthread_mutex_init(&mut, NULL);
    if(pthread_create (&th,NULL,lcdRoutine,NULL)<0)  //this is the function used for display the
                                                    //lcd screen
          {
			printf("Problem with the thread of the screen\n");
            exit(0);
          }
  }
 
/*******************************************************************/
  
  try {
    
    
    //cout << "*** running automate *** \n" << "*** p : print status ***\n"  << "*** q : quit ! ***\n";
    send_mail("automate dmarr",c);
    
    //init some ouputs
    // for updating k8055
    a.getSkimmer()->changeState(1,c,k,cfg);
    // init counter if used
    if (cfg.FLOW !=0) k.resetcounter(cfg.FLOW);  

	// init correction ph sensor
	a.getphsonde()->set_correction(c.get_calibre());
    // init timer of the rah pump
    time ( &(a.getRah()->timer));

    // init timer of the hqi
    time ( &(a.getHqi()->timer));
    
    while(ch!= 'q' && a.getstart==0)
      {
	//k.print_status(k.uh,cfg);
	//******************************************************************* 
	    // checking if the automaton must be restarted 
	a.getstart=c.get_start();
	a.restart(c);
	
	//******************************************************************* 	
	    //getting input states
	a.getInputState(k,cfg);
	//******************************************************************* 
	    // getting analogic input 1 => temperature 
	    t2= k.readAnalog(INP_TEMP);
	te[count] =(101*t2/256)-23;
     	count++;
	if (count==20) 
	  {
	    
	    tempe=round(a.getTemperature()->give(te)*100)/100;
	    if(tempe<20 || tempe>40) {c.insert_log("erreur!!!!!");ch='q';} //error quit
	    else temperature=tempe;
	    count=0;
	    if(cfg.LCD)
	      {
		pthread_mutex_lock(&mut);
		sprintf(temp,"%g",temperature);
		dataVal.set_temperature(temp);
		pthread_mutex_unlock(&mut);
	      }
	    else {} //ici il faudrait dtruire le thread de la routine de l'cran 
	  }
	
	//*******************************************************************
	    // get the forced status of output 0:normal 1:stop forced 2:run forced 
	c.get_forced(cfg);
	
	//*******************************************************************
	    // state of hqi changing ?
	if (cfg.OUTHQI!=0)
	{
	  st=a.getHqi()->get_State("hqi",c);
	  a.getHqi()->changing(st,temperature,c,k,cfg);
	}
	
	//*******************************************************************
	    // state of blue changing ?
	if (cfg.OUTBLUE!=0)
	{
		st=a.getBlue()->get_State("blue",c);
		
		a.getBlue()->changing(st,c,k,cfg);
	}
	
	//*******************************************************************
	    //state of pumps 
	    //depends of param obj and of input 1 ( physical button)
	if (cfg.OUTPUMP!=0)
	{
		st=a.getPump()->get_State("pump",c);
		if(cfg.LCD)
		  {
		    pthread_mutex_lock(&mut);
		    if (a.getPump()->getState()==0)
		      dataVal.set_gdespompes("off");
		    else dataVal.set_gdespompes("on");
		    pthread_mutex_unlock(&mut);
		  }
		a.getPump()->changing (st,c,k,cfg);
		
	}
	//*******************************************************************
	    //enabling / stopping skimmer pump
	    //stopping skimmer if the high captor is enabled
	  if (cfg.OUTSKIMMER!=0)
	    {
		st=a.getSkimmer()->get_State("skimmer",c);
	      a.getSkimmer()->changing(st,a.getOsmolation()->getCaptor_high(),c,k,cfg);
	      if(cfg.LCD)
		{
		  pthread_mutex_lock(&mut);
		  if (a.getSkimmer()->getState()==0)
		      dataVal.set_ecumeur("off");
		  else dataVal.set_ecumeur("on");
		  pthread_mutex_unlock(&mut);
		}
	    }
	
	//*******************************************************************
	    //state of osmolations 
	  if (cfg.OUTOSMOLATION!=0){
	    
	    a.getOsmolation()->changing(c,k,cfg);
	   
	 if(cfg.LCD)
		{
		  pthread_mutex_lock(&mut);
		  if (a.getOsmolation()->getState()==0)
		    dataVal.set_osmolation("off");
		  else dataVal.set_osmolation("on");
		 if(a.getOsmolation()->getEmpty_reserv()==0)
		  dataVal.set_reserve("ok");
		    else dataVal.set_reserve("vide");
		
		pthread_mutex_unlock(&mut);
		}
  }
	
	//*******************************************************************

//*******************************************************************
	    //state of ventilator
	  if (cfg.OUTVENT!=0)
		{
	        a.getVentilator()->changing(temperature,c,k,cfg);
		}
	
	//*******************************************************************
	    //temperature state
	    // maj current temperature
	if (cfg.OUTTEMP!=0)
	{
		c.insert_current_temperature(temperature);
		a.getTemperature()->tempState(temperature,c,k,cfg);
		if(cfg.LCD)
		{
		  pthread_mutex_lock(&mut);
		  if (a.getTemperature()->getState()==0)
		    dataVal.set_chauffage("off");
		  else dataVal.set_chauffage("on");
		  pthread_mutex_unlock(&mut);
		}
	}
	//*******************************************************************
                  //ph state
                  // maj current ph
                 if (cfg.PH != 0) {
                 ph=a.getphsonde()->read_ph(c);
                  //cout << ph << endl;
                  }
                 
      //*******************************************************************
	    // some operations depending on time
    	time (&(a.getTemperature()->timer));
	s=ctime(&(a.getTemperature()->timer)); 
	s2=s.substr(s.find(":",0)+1,2);
	st=atoi(s2.c_str()); // minute
	s2=s.substr(s.find(":",0)-2,2); 
	sh=atoi(s2.c_str()); //hour
	
	if (st==0 && new_m==false) 
	  {
	    new_m=true;
	    if (cfg.FLOWTYPE==1) //if flowmeter used without the osmolation module 
		{
		cout << "flow " << flow << endl;
		flow=(double)k.readcounter(cfg.FLOW);
		c.insert_osmolation((flow)/cfg.FLOWIMPULSE);
		k.resetcounter(cfg.FLOW);
		cout << "flow " << flow << endl;
		}
		if (sh==0)
	      {
		//new day !!!
		a.getHqi()->setStop(false); // re-enable HQI 
		c.maj_log(); // delete logs of the previous month
		c.maj_water(); //
		m=s.substr(8,2);
		day=atoi(m.c_str());
		if(day==1) {
			c.maj_temperature();  // new month => min and max temp deleted in temperature table
			c.maj_ph(); // new month => min and max temp deleted in ph table
			     }
	      }
	    
	    c.insert_temperature(temperature);
	    if (a.getphsonde()->get_error()==false) c.insert_ph(a.getphsonde()->get_ph());	
	    c.save(); // save current bd every hour
	  }
	if (st==1 && new_m==true)
	  new_m=false;
	
        
	
	
	
	//*******************************************************************
	    // rah pump state
	if (cfg.OUTRAH!=0)
	{
		st=a.getRah()->get_State("rah",c);
		if(cfg.LCD)
		  {
		    pthread_mutex_lock(&mut);
		    if (a.getRah()->getState()==0)
		      dataVal.set_rah("off");
		    else dataVal.set_rah("on");
		    pthread_mutex_unlock(&mut);
		  }
		a.getRah()->changing(st,c,k,cfg);
	}
	//*******************************************************************
	    // state of peristatic pump
	if (cfg.OUTPERI!=0)
	{
		st=a.getPeristatic()->get_State("pdoseuse",c);
		 if(cfg.LCD)
		   {
		     pthread_mutex_lock(&mut);
		     if (a.getPeristatic()->getState()==0)
	    		dataVal.set_pompes("off");
		     else dataVal.set_pompes("on");
		     pthread_mutex_unlock(&mut);
		   }
		 a.getPeristatic()->changing(st,c,k,cfg);
	}
	//*******************************************************************
	    
      }//endwhile
    
    c.set_stop();
    c.close();
	
    
    
  }
  catch (mysqlpp::BadQuery& er) {
    // handle any connection or query errors that may come up
    cout << "Error: mysql";
    return -1;exit(0);
  }
  
return 0;
}

